@@ -2,7 +2,7 @@
 # least squares fits.
 #
 # Written by Konrad Hinsen <hinsen@cnrs-orleans.fr>
-# last revision: 2002-11-18
+# last revision: 2004-4-7
 #
 
 import Numeric, LinearAlgebra
@@ -24,7 +24,8 @@ def _chiSquare(model, parameters, data):
 	alpha = alpha + d[:,Numeric.NewAxis]*d
     return chi_sq, alpha
 
-def leastSquaresFit(model, parameters, data, max_iterations=None):
+def leastSquaresFit(model, parameters, data, max_iterations=None,
+                    stopping_limit = 0.005):
     """General non-linear least-squares fit using the
     Levenberg-Marquardt algorithm and automatic derivatives.
 
@@ -72,7 +73,7 @@ def leastSquaresFit(model, parameters, data, max_iterations=None):
 	    l = 10.*l
 	else:
 	    l = 0.1*l
-	    if chi_sq[0] - next_chi_sq[0] < 0.005: break
+	    if chi_sq[0] - next_chi_sq[0] < stopping_limit: break
 	    p = next_p
 	    chi_sq = next_chi_sq
 	    alpha = next_alpha
