@@ -1144,7 +1144,7 @@ if __name__ == '__main__':
 
     if 1:
 
-        s = Structure('~/detached/okb.pdb')
+        s = Structure('~/1C1C.pdb')
         #s = Structure('./3lzt.pdb')
         #s = Structure('~/1tka.pdb')
         print s
