@@ -1,7 +1,7 @@
 # Physical quantities with units
 #
 # Written by Konrad Hinsen <hinsen@cnrs-orleans.fr>
-# last revision: 1999-7-23
+# last revision: 2002-4-9
 #
 
 # hacked 1998/09/28 GPW: now removes __args__ from local dict after eval
@@ -73,7 +73,7 @@ class PhysicalQuantity:
 	    self.value = string.atof(match.group(0))
 	    self.unit = _findUnit(s[len(match.group(0)):])
 
-    _number = re.compile('[+-]?[0-9]+(\\.[0-9]*)?([eE][+-]?[0-9]?)?')
+    _number = re.compile('[+-]?[0-9]+(\\.[0-9]*)?([eE][+-]?[0-9]+)?')
 
     def __str__(self):
 	return str(self.value) + ' ' + self.unit.name()
