@@ -5,3 +5,11 @@ import os, sys
 sys.path.append(os.path.join(os.path.split(__file__)[0], sys.platform))
 del os
 del sys
+
+#
+# New exception class
+#
+class IterationCountExceededError(ValueError):
+
+    pass
+
