@@ -4,7 +4,7 @@
  * Written by Konrad Hinsen <hinsen@cnrs-orleans.fr>
  *        and Jakob Schiotz <schiotz@fysik.dtu.dk>
  *        and Ciro Cattuto <ciro@prosa.it>
- * last revision: 2001-3-29
+ * last revision: 2004-3-29
  */
 
 
@@ -65,6 +65,7 @@ mpi_op(PyMPIOperationObject *op)
   return op->mpi_op;
 }
 
+/* Doubles the count for sending/receiving complex numbers. */
 static int
 mpi_count_factor(int py_type)
 {
@@ -88,6 +89,7 @@ PyMPI_SetError(int errcode, char *funcname)
   return NULL;
 }
 
+
 /***************************************/
 /* MPI Operation object implementation */
 /***************************************/
@@ -128,7 +130,7 @@ PyMPIOperation_repr(PyMPIOperationObject *self)
 }
 
 statichere PyTypeObject PyMPIOperation_Type = {
-	PyObject_HEAD_INIT(&PyType_Type)
+	PyObject_HEAD_INIT(NULL)
 	0,				/*ob_size*/
 	"PyMPIOperation",		/*tp_name*/
 	sizeof(PyMPIOperationObject),	/*tp_basicsize*/
@@ -145,7 +147,7 @@ statichere PyTypeObject PyMPIOperation_Type = {
 	0,				/*tp_as_mapping*/
 	0,				/*tp_hash*/
 	0,				/*tp_call*/
-	0				/*tp_str*/
+	0				  /*tp_str*/
 };
 
 /******************************************/
@@ -211,6 +213,74 @@ PyMPICommunicator_duplicate(PyMPICommunicatorObject *self, PyObject *args)
 }
 
 
+/* Subset */
+
+static PyObject *
+PyMPI_SubsetCommunicator(PyMPICommunicatorObject *comm, PyArrayObject *array)
+{
+  MPI_Group group;
+  MPI_Group newgroup;
+  MPI_Comm new;
+  int *ranks;
+  int dimension, i1, i2;
+
+  if (MPI_Comm_group(comm->handle, &group) != MPI_SUCCESS) {
+    PyErr_SetString(PyExc_MPIError, "MPI_Comm_group failed");
+    return NULL;
+  }
+  dimension = array->dimensions[0];
+  ranks = (int*)(array->data);
+  for (i1 = 0; i1 < dimension; i1++) {
+    if (ranks[i1] < 0 || ranks[i1] >= comm->size) {
+      PyErr_SetString(PyExc_MPIError, "invalid MPI rank");
+      return NULL;
+    }
+    for (i2 = 0; i2 < i1; i2++)
+      if (ranks[i1] == ranks[i2]) {
+	PyErr_SetString(PyExc_MPIError, "duplicated MPI rank");
+	return NULL;
+      }
+  }
+  if (MPI_Group_incl(group, dimension, ranks, &newgroup) != MPI_SUCCESS) {
+    PyErr_SetString(PyExc_MPIError, "MPI_Group_incl failed");
+    return NULL;
+  }
+  if (MPI_Comm_create(comm->handle, newgroup, &new) != MPI_SUCCESS) {
+    PyErr_SetString(PyExc_MPIError, "MPI_Comm_create failed");
+    return NULL;
+  }
+  if (MPI_Group_free(&newgroup) != MPI_SUCCESS) {
+    PyErr_SetString(PyExc_MPIError, "MPI_Group_free failed");
+    return NULL;
+  }
+  if (MPI_Group_free(&group) != MPI_SUCCESS) {
+    PyErr_SetString(PyExc_MPIError, "MPI_Group_free failed");
+    return NULL;
+  }
+  if (new == MPI_COMM_NULL) {
+    Py_INCREF(Py_None);
+    return Py_None;
+  }
+  else
+    return (PyObject *)newPyMPICommunicatorObject(new);
+}
+
+static PyObject *
+PyMPICommunicator_subset(PyMPICommunicatorObject *self, PyObject *args)
+{
+  PyObject *ranks;
+  PyObject *new;
+  if (!PyArg_ParseTuple(args, "O", &ranks))
+    return NULL;
+  ranks = PyArray_ContiguousFromObject(ranks, PyArray_INT, 1, 1);
+  if (ranks == NULL)
+    return NULL;
+  new = PyMPI_SubsetCommunicator(self, (PyArrayObject *)ranks);
+  Py_DECREF(ranks);
+  return new;
+}
+
+
 /* Barrier */
 
 static int
@@ -287,7 +357,7 @@ PyMPI_SendString(PyMPICommunicatorObject *comm, PyStringObject *string,
 		int dest, int tag)
 {
   char *data;
-  int error;
+  int error, count;
 
   if (tag < 0 || tag > max_tag) {
     PyErr_SetString(PyExc_MPIError, "invalid MPI tag");
@@ -298,8 +368,9 @@ PyMPI_SendString(PyMPICommunicatorObject *comm, PyStringObject *string,
     return -1;
   }
   data = PyString_AsString((PyObject *)string);
+  count = PyString_GET_SIZE(string);
   Py_BEGIN_ALLOW_THREADS;
-  error = PyMPI_Send(comm, data, PyArray_CHAR, strlen(data), dest, tag);
+  error = PyMPI_Send(comm, data, PyArray_CHAR, count, dest, tag);
   Py_END_ALLOW_THREADS;
   if (error != MPI_SUCCESS) {
     PyErr_SetString(PyExc_MPIError, "MPI_Send failed");
@@ -579,11 +650,11 @@ PyMPICommunicator_receive(PyMPICommunicatorObject *self, PyObject *args)
 static PyObject *
 PyMPICommunicator_receiveString(PyMPICommunicatorObject *self, PyObject *args)
 {
-  PyObject *source_ob, *tag_ob;
+  PyObject *source_ob = Py_None, *tag_ob = Py_None, *return_ob;
   PyObject *string;
   int source, tag;
 
-  if (!PyArg_ParseTuple(args, "OO", &source_ob, &tag_ob))
+  if (!PyArg_ParseTuple(args, "|OO", &source_ob, &tag_ob))
     return NULL;
   if (source_ob == Py_None)
     source = MPI_ANY_SOURCE;
@@ -606,7 +677,10 @@ PyMPICommunicator_receiveString(PyMPICommunicatorObject *self, PyObject *args)
   if (string == NULL)
     return NULL;
 
-  return Py_BuildValue("Oii", string, source, tag);
+  return_ob = Py_BuildValue("Oii", string, source, tag);
+  Py_DECREF(string);
+  return return_ob;
+
 }
 
 /* Nonblocking receive */
@@ -1004,6 +1078,7 @@ PyMPICommunicator_allreduce(PyMPICommunicatorObject *self, PyObject *args)
 
 static PyMethodDef PyMPICommunicator_methods[] = {
   {"duplicate", (PyCFunction)PyMPICommunicator_duplicate, 1},
+  {"subset", (PyCFunction)PyMPICommunicator_subset, 1},
   {"send", (PyCFunction)PyMPICommunicator_send, 1},
   {"nonblocking_send", (PyCFunction)PyMPICommunicator_nonblocking_send, 1},
   {"nonblockingSend", (PyCFunction)PyMPICommunicator_nonblocking_send, 1},
@@ -1040,7 +1115,7 @@ PyMPICommunicator_getattr(PyMPICommunicatorObject *self, char *name)
 statichere PyTypeObject PyMPICommunicator_Type = {
 	/* The ob_type field must be initialized in the module init function
 	 * to be portable to Windows without using C++. */
-	PyObject_HEAD_INIT(NULL)   /**** BUT THAT IS NOT DONE ****/
+	PyObject_HEAD_INIT(NULL)
 	0,			/*ob_size*/
 	"PyMPICommunicator",			/*tp_name*/
 	sizeof(PyMPICommunicatorObject),	/*tp_basicsize*/
@@ -1189,7 +1264,7 @@ PyMPIRequest_getattr(PyMPICommunicatorObject *self, char *name)
 }
 
 statichere PyTypeObject PyMPIRequest_Type = {
-	PyObject_HEAD_INIT(&PyType_Type)
+	PyObject_HEAD_INIT(NULL)
 	0,				/*ob_size*/
 	"PyMPIRequest",			/*tp_name*/
 	sizeof(PyMPIRequestObject),	/*tp_basicsize*/
@@ -1242,26 +1317,35 @@ static PyMethodDef mpi_methods[] = {
 	PyDict_SetItemString(d, op_name, op); \
 	} while (0)
 
-void
+DL_EXPORT(void)
 initScientific_mpi(void)
 {
   PyObject *m, *d, *world;
   static void *PyMPI_API[PyMPI_API_pointers];
   int mpi_init_flag;
 
+  /* Set type of locally created static objects. */
+  PyMPIOperation_Type.ob_type    = &PyType_Type;
+  PyMPICommunicator_Type.ob_type = &PyType_Type;
+  PyMPIRequest_Type.ob_type      = &PyType_Type;
+
   /* Create the module */
   m = Py_InitModule("Scientific_mpi", mpi_methods);
   d = PyModule_GetDict(m);
 
-  /* Import the array module */
-#ifdef import_array
-  import_array();
-#endif
-
   /* Initialize C API pointer array and store in module */
   set_PyMPI_API_pointers();
   PyDict_SetItemString(d, "_C_API", PyCObject_FromVoidPtr(PyMPI_API, NULL));
 
+  /* Import the array module */
+#ifdef import_array
+  import_array();
+  if (PyErr_Occurred()) {
+    PyErr_SetString(PyExc_ImportError, "Can\'t import Numeric.");
+    return;
+  }
+#endif
+
   /* Check that MPI has been initialized */
   if (MPI_Initialized(&mpi_init_flag) != MPI_SUCCESS || !mpi_init_flag) {
     Py_INCREF(Py_None);
@@ -1295,10 +1379,10 @@ initScientific_mpi(void)
 
   /* Check for errors */
   if (PyErr_Occurred())
-    Py_FatalError("can't initialize module Scientific_mpi");
+    PyErr_SetString(PyExc_ImportError, "Can\'t initialize module.");
 }
 
-  
+
 /* Keep Konrad Hinsens indentation style when using cc mode in (x)emacs. */
 /* Local Variables: */
 /* c-basic-offset: 2 */
