@@ -2,9 +2,12 @@
  * Objects representing netCDF files and variables.
  *
  * Written by Konrad Hinsen
- * last revision: 2001-1-8
+ * last revision: 2004-11-18
  */
 
+#ifdef _WIN32
+#define DLL_NETCDF
+#endif
 
 #include "Python.h"
 #include "Numeric/arrayobject.h"
@@ -13,14 +16,13 @@
 #define _NETCDF_MODULE
 #include "Scientific/netcdfmodule.h"
 
-
 staticforward int
 netcdf_file_init(PyNetCDFFileObject *self);
 staticforward PyNetCDFVariableObject *
 netcdf_variable_new(PyNetCDFFileObject *file, char *name, int id, int type,
 		    int ndims, int *dimids, int nattrs);
 
-/* Lock which grants access to netCDF routines (netCDF isn't thread-safe) */
+/* Lock granting access to netCDF routines (netCDF isn't thread-safe) */
 
 #ifdef WITH_THREAD
 
@@ -36,46 +38,110 @@ PyThread_type_lock netCDF_lock;
 
 #endif
 
-/* Error object and error messages for netcdf-specific errors */
-
-static char *netcdf_errors[] = {
-  "No Error",
-  "Not a netcdf id",
-  "Too many netcdfs open",
-  "netcdf file exists && NC_NOCLOBBER",
-  "Invalid Argument",
-  "Write to read only",
-  "Operation not allowed in data mode",
-  "Operation not allowed in define mode",
-  "Coordinates out of Domain",
-  "MAX_NC_DIMS exceeded",
-  "String match to name in use",
-  "Attribute not found",
-  "MAX_NC_ATTRS exceeded",
-  "Not a netcdf data type",
-  "Invalid dimension id",
-  "NC_UNLIMITED in the wrong index",
-  "MAX_NC_VARS exceeded",
-  "Variable not found",
-  "Action prohibited on NC_GLOBAL varid",
-  "Not a netcdf file",
-  "In Fortran, string too short",
-  "MAX_NC_NAME exceeded",
-  "NC_UNLIMITED size already in use", /* 22 */
-  "", "", "", "", "", "", "", "", "",
-  "XDR error" /* 32 */
-};
-
 /* Set error string */
 static void
 netcdf_seterror(void)
 {
-  if (ncerr != 0) {
-    char *error = "Unknown error";
-    if (ncerr > 0 && ncerr <= 32)
-      error = netcdf_errors[ncerr];
-    PyErr_SetString(PyExc_IOError, error);
+  char *error;
+  switch (ncerr) {
+  case NC_NOERR:
+    error = "No error";
+    break;
+  case NC_EBADID:
+    error = "Not a netCDF id";
+    break;
+  case NC_ENFILE:
+    error = "Too many netCDF files open";
+    break;
+  case NC_EEXIST:
+    error = "netCDF file exists && NC_NOCLOBBER";
+    break;
+  case NC_EINVAL:
+    error = "Invalid argument";
+    break;
+  case NC_EPERM:
+    error = "Write to read only";
+    break;
+  case NC_ENOTINDEFINE:
+    error = "Operation not allowed in data mode";
+    break;
+  case NC_EINDEFINE:
+    error = "Operation not allowed in define mode";
+    break;
+  case NC_EINVALCOORDS:
+    error = "Index exceeds dimension bound";
+    break;
+  case NC_EMAXDIMS:
+    error = "NC_MAX_DIMS exceeded";
+    break;
+  case NC_ENAMEINUSE:
+    error = "String match to name in use";
+    break;
+  case NC_ENOTATT:
+    error = "Attribute not found";
+    break;
+  case NC_EMAXATTS:
+    error = "NC_MAX_ATTRS exceeded";
+    break;
+  case NC_EBADTYPE:
+    error = "Not a netCDF data type or _FillValue type mismatch";
+    break;
+  case NC_EBADDIM:
+    error = "Invalid dimension id or name";
+    break;
+  case NC_EUNLIMPOS:
+    error = "NC_UNLIMITED in the wrong index";
+    break;
+  case NC_EMAXVARS:
+    error = "NC_MAX_VARS exceeded";
+    break;
+  case NC_ENOTVAR:
+    error = "Variable not found";
+    break;
+  case NC_EGLOBAL:
+    error = "Action prohibited on NC_GLOBAL varid";
+    break;
+  case NC_ENOTNC:
+    error = "Not a netCDF file";
+    break;
+  case NC_ESTS:
+    error = "In Fortran, string too short";
+    break;
+  case NC_EMAXNAME:
+    error = "NC_MAX_NAME exceeded";
+    break;
+  case NC_EUNLIMIT:
+    error = "NC_UNLIMITED size already in use";
+    break;
+  case NC_ENORECVARS:
+    error = "nc_rec op when there are no record vars";
+    break;
+  case NC_ECHAR:
+    error = "Attempt to convert between text & numbers";
+    break;
+  case NC_EEDGE:
+    error = "Edge+start exceeds dimension bound";
+    break;
+  case NC_ESTRIDE:
+    error = "Illegal stride";
+    break;
+  case NC_EBADNAME:
+    error = "Attribute or variable name contains illegal characters";
+    break;
+  case NC_ERANGE:
+    error = "Numeric conversion not representable";
+    break;
+  case NC_ENOMEM:
+    error = "Memory allocation (malloc) failure";
+    break;
+  case NC_EXDR:
+    error = "XDR error";
+    break;
+  default:
+    error = "Unknown error";
+    break;
   }
+  PyErr_SetString(PyExc_IOError, error);
 }
 
 static void
@@ -88,6 +154,7 @@ netcdf_signalerror(int code)
     sprintf(buffer, "netcdf: %s", nc_strerror(code));
     release_netCDF_lock();
     Py_END_ALLOW_THREADS;
+    fprintf(stderr, buffer); printf("\n");
     PyErr_SetString(PyExc_IOError, buffer);
  }
 }
@@ -109,6 +176,108 @@ int data_types[] = {-1,  /* not used */
 		    PyArray_DOUBLE  /* 64-bit IEEE float */
 };
 
+/* Generic data type functions, similar to those in the netCDF 2 interface. */
+
+static int
+nc_put_att_any(int ncid, int varid, const char *name,
+	       nc_type xtype, size_t len, const void *data)
+{
+  switch (xtype) {
+  case NC_BYTE:
+    return nc_put_att_uchar(ncid, varid, name, xtype, len,
+			    (unsigned char *)data);
+    break;
+  case NC_CHAR:
+    return nc_put_att_text(ncid, varid, name, len,
+			   (char *)data);
+    break;
+  case NC_SHORT:
+    return nc_put_att_short(ncid, varid, name, xtype, len,
+			    (short *)data);
+    break;
+  case NC_INT:
+    return nc_put_att_int(ncid, varid, name, xtype, len,
+			   (int *)data);
+    break;
+  case NC_FLOAT:
+    return nc_put_att_float(ncid, varid, name, xtype, len,
+			    (float *)data);
+    break;
+  case NC_DOUBLE:
+    return nc_put_att_double(ncid, varid, name, xtype, len,
+			     (double *)data);
+    break;
+  default:
+    return NC_EINVAL;
+  }
+}
+
+static int
+nc_put_var1_any(int ncid, int varid, nc_type xtype, const size_t *indexp,
+		const void *data)
+{
+  switch (xtype) {
+  case NC_BYTE:
+    return nc_put_var1_uchar(ncid, varid, indexp, (unsigned char *)data);
+    break;
+  case NC_CHAR:
+    return nc_put_var1_text(ncid, varid, indexp, (char *)data);
+    break;
+  case NC_SHORT:
+    return nc_put_var1_short(ncid, varid, indexp, (short *)data);
+    break;
+  case NC_INT:
+    return nc_put_var1_int(ncid, varid, indexp, (int *)data);
+    break;
+  case NC_FLOAT:
+    return nc_put_var1_float(ncid, varid, indexp, (float *)data);
+    break;
+  case NC_DOUBLE:
+    return nc_put_var1_double(ncid, varid, indexp, (double *)data);
+    break;
+  default:
+    return NC_EINVAL;
+  }
+}
+
+static int
+nc_put_vars_any(int ncid, int varid, nc_type xtype, const size_t start[],
+		const size_t count[], const ptrdiff_t stride[],
+		const void *data)
+{
+  switch (xtype) {
+  case NC_BYTE:
+    return nc_put_vars_uchar(ncid, varid, start, count, stride,
+			     (unsigned char *)data);
+    break;
+  case NC_CHAR:
+    return nc_put_vars_text(ncid, varid, start, count, stride,
+			     (char *)data);
+    break;
+  case NC_SHORT:
+    return nc_put_vars_short(ncid, varid, start, count, stride,
+			     (short *)data);
+    break;
+    break;
+  case NC_INT:
+    return nc_put_vars_int(ncid, varid, start, count, stride,
+			   (int *)data);
+    break;
+    break;
+  case NC_FLOAT:
+    return nc_put_vars_float(ncid, varid, start, count, stride,
+			     (float *)data);
+    break;
+    break;
+  case NC_DOUBLE:
+    return nc_put_vars_double(ncid, varid, start, count, stride,
+			      (double *)data);
+    break;
+  default:
+    return NC_EINVAL;
+  }
+}
+
 /* Utility functions */
 
 static void
@@ -118,9 +287,9 @@ define_mode(PyNetCDFFileObject *file, int define_flag)
     Py_BEGIN_ALLOW_THREADS;
     acquire_netCDF_lock();
     if (file->define)
-      ncendef(file->id);
+      nc_enddef(file->id);
     else
-      ncredef(file->id);
+      nc_redef(file->id);
     release_netCDF_lock();
     file->define = define_flag;
     Py_END_ALLOW_THREADS;
@@ -161,7 +330,7 @@ typecode(int type)
   return t;
 }
 
-static int
+static nc_type
 netcdf_type_from_code(char code)
 {
   int type;
@@ -178,7 +347,7 @@ netcdf_type_from_code(char code)
     break;
   case 'i':
   case 'l':
-    type = NC_LONG;
+    type = NC_INT;
     break;
   case 'f':
     type = NC_FLOAT;
@@ -192,6 +361,37 @@ netcdf_type_from_code(char code)
   return type;
 }
 
+static int
+netcdf_type_from_type(char array_type)
+{
+  int type;
+  switch(array_type) {
+  case PyArray_CHAR:
+    type = NC_CHAR;
+    break;
+  case PyArray_UBYTE:
+  case PyArray_SBYTE:
+    type = NC_BYTE;
+    break;
+  case PyArray_SHORT:
+    type = NC_SHORT;
+    break;
+  case PyArray_INT:
+  case PyArray_LONG:
+    type = NC_INT;
+    break;
+  case PyArray_FLOAT:
+    type = NC_FLOAT;
+    break;
+  case PyArray_DOUBLE:
+    type = NC_DOUBLE;
+    break;
+  default:
+    type = 0;
+  }
+  return type;
+}
+
 
 static void
 collect_attributes(int fileid, int varid, PyObject *attributes, int nattrs)
@@ -274,8 +474,7 @@ set_attribute(int fileid, int varid, PyObject *attributes,
       int type = netcdf_type_from_code(array->descr->type);
       if (data_types[type] != array->descr->type_num) {
 	PyArrayObject *array2 = (PyArrayObject *)
-	  PyArray_ContiguousFromObject((PyObject *)array,
-				       data_types[type], 0, 1);
+	  PyArray_Cast((PyObject *)array, data_types[type]);
 	Py_DECREF(array);
 	array = array2;
 	if (array == NULL)
@@ -283,11 +482,11 @@ set_attribute(int fileid, int varid, PyObject *attributes,
       }
       Py_BEGIN_ALLOW_THREADS;
       acquire_netCDF_lock();
-      ret = ncattput(fileid, varid, name, type, len, array->data);
+      ret = nc_put_att_any(fileid, varid, name, type, len, array->data);
       release_netCDF_lock();
       Py_END_ALLOW_THREADS;
-      if (ret == -1) {
-	netcdf_seterror();
+      if (ret != NC_NOERR) {
+	netcdf_signalerror(ret);
 	return -1;
       }
       PyDict_SetItemString(attributes, name, (PyObject *)array);
@@ -600,7 +799,7 @@ PyNetCDFFile_CreateVariable(PyNetCDFFileObject *file, char *name, int typecode,
 	return NULL;
       }
     }
-    ntype = netcdf_type_from_code(typecode);
+    ntype = netcdf_type_from_code((char)typecode);
     Py_BEGIN_ALLOW_THREADS;
     acquire_netCDF_lock();
     ret = nc_def_var(file->id, name, ntype, ndim, dimids, &i);
@@ -955,10 +1154,10 @@ netcdf_variable_new(PyNetCDFFileObject *file, char *name, int id, int type,
     Py_BEGIN_ALLOW_THREADS;
     acquire_netCDF_lock();
     ncinquire(file->id, NULL, NULL, NULL, &recdim);
-    self->dimensions = (long *)malloc(ndims*sizeof(long));
+    self->dimensions = (size_t *)malloc(ndims*sizeof(size_t));
     if (self->dimensions != NULL) {
       for (i = 0; i < ndims; i++)
-	ncdiminq(file->id, dimids[i], NULL, &self->dimensions[i]);
+	nc_inq_dimlen(file->id, dimids[i], &self->dimensions[i]);
       if (ndims > 0 && self->dimids[0] == self->file->recdim)
 	self->unlimited = 1;
     }
@@ -1037,7 +1236,7 @@ PyNetCDFVariable_GetRank(PyNetCDFVariableObject *var)
   return var->nd;
 }
 
-static long *
+static size_t *
 PyNetCDFVariable_GetShape(PyNetCDFVariableObject *var)
 {
   int i;
@@ -1045,7 +1244,7 @@ PyNetCDFVariable_GetShape(PyNetCDFVariableObject *var)
     Py_BEGIN_ALLOW_THREADS;
     acquire_netCDF_lock();
     for (i = 0; i < var->nd; i++)
-      ncdiminq(var->file->id, var->dimids[i], NULL, &var->dimensions[i]);
+      nc_inq_dimlen(var->file->id, var->dimids[i], &var->dimensions[i]);
     release_netCDF_lock();
     Py_END_ALLOW_THREADS;
     return var->dimensions;
@@ -1284,7 +1483,6 @@ PyNetCDFVariable_ReadAsString(PyNetCDFVariableObject *self)
     return NULL;
   }
   if (check_if_open(self->file, -1)) {
-    long zero = 0;
     int ret;
     char *temp;
     PyObject *string;
@@ -1294,11 +1492,11 @@ PyNetCDFVariable_ReadAsString(PyNetCDFVariableObject *self)
       return (PyStringObject *)PyErr_NoMemory();
     Py_BEGIN_ALLOW_THREADS;
     acquire_netCDF_lock();
-    ret = ncvarget(self->file->id, self->id, &zero, self->dimensions, temp);
+    ret = nc_get_var_text(self->file->id, self->id, temp);
     release_netCDF_lock();
     Py_END_ALLOW_THREADS;
-    if (ret == -1) {
-      netcdf_seterror();
+    if (ret != NC_NOERR) {
+      netcdf_signalerror(ret);
       string = NULL;
     }
     else {
@@ -1312,6 +1510,9 @@ PyNetCDFVariable_ReadAsString(PyNetCDFVariableObject *self)
     return NULL;
 }
 
+#include "time.h"
+
+
 static int
 PyNetCDFVariable_WriteArray(PyNetCDFVariableObject *self,
 			    PyNetCDFIndex *indices, PyObject *value)
@@ -1375,35 +1576,35 @@ PyNetCDFVariable_WriteArray(PyNetCDFVariableObject *self,
 							0, d);
   if (array != NULL) {
     if (self->nd == 0) {
-      long zero = 0;
-      int ret;
+      size_t zero = 0;
       Py_BEGIN_ALLOW_THREADS;
       acquire_netCDF_lock();
-      ret = ncvarput1(self->file->id, self->id, &zero, array->data);
+      error = nc_put_var1_any(self->file->id, self->id,
+			      netcdf_type_from_type(self->type), &zero,
+			      array->data);
       release_netCDF_lock();
       Py_END_ALLOW_THREADS;
-      if (ret == -1) {
-	netcdf_seterror();
+      if (error != NC_NOERR) {
+	netcdf_signalerror(ret);
 	ret = -1;
       }
     }
     else {
-      long *start;
-      long *count, *count1;
-      long *stride;
-      long *current;
+      size_t *start;
+      size_t *count, *count1;
+      ptrdiff_t *stride;
+      size_t *current;
       char *loop;
       long repeat = 1;
-      int elim, lastloop;
-      start = (long *)malloc(self->nd*sizeof(long));
-      count = (long *)malloc(self->nd*sizeof(long));
-      count1 = (long *)malloc(self->nd*sizeof(long));
-      stride = (long *)malloc(self->nd*sizeof(long));
-      current = (long *)malloc(self->nd*sizeof(long));
+      int lastloop;
+      start = (size_t *)malloc(self->nd*sizeof(size_t));
+      count = (size_t *)malloc(self->nd*sizeof(size_t));
+      count1 = (size_t *)malloc(self->nd*sizeof(size_t));
+      stride = (ptrdiff_t *)malloc(self->nd*sizeof(ptrdiff_t));
+      current = (size_t *)malloc(self->nd*sizeof(size_t));
       loop = (char *)malloc(self->nd*sizeof(char));
       if (start != NULL && count != NULL && count1 != NULL
 	  && stride != NULL && current != NULL && loop != NULL) {
-	elim = 0;
 	for (i = 0; i < self->nd; i++) {
 	  start[i] = indices[i].start;
 	  stride[i] = indices[i].stride;
@@ -1411,8 +1612,6 @@ PyNetCDFVariable_WriteArray(PyNetCDFVariableObject *self,
 	  count1[i] = count[i];
 	  current[i] = 0;
 	  loop[i] = 0;
-	  if (indices[i].item)
-	    elim++;
 	}
 	for (i = array->nd-1, j = self->nd-1; i >= 0 && j >= 0; i--, j--) {
 	  while (j >= 0 && indices[j].item)
@@ -1441,18 +1640,15 @@ PyNetCDFVariable_WriteArray(PyNetCDFVariableObject *self,
 	  ret = -1;
 	if (ret == -1)
 	  PyErr_SetString(PyExc_ValueError, "shapes are not aligned");
-	while (repeat-- && ret == 0) {
-	  int ret;
-	  Py_BEGIN_ALLOW_THREADS;
-	  acquire_netCDF_lock();
-	  ret = ncvarputg(self->file->id, self->id, start, count1, stride,
-			  NULL, array->data);
-	  release_netCDF_lock();
-	  Py_END_ALLOW_THREADS;
-	  if (ret == -1) {
-	    netcdf_seterror();
-	    ret = -1;
-	  }
+	Py_BEGIN_ALLOW_THREADS;
+	acquire_netCDF_lock();
+	error = NC_NOERR;
+	while (repeat--) {
+	  error = nc_put_vars_any(self->file->id, self->id,
+				  netcdf_type_from_type(self->type),
+				  start, count1, stride, array->data);
+	  if (error != NC_NOERR)
+	    break;
 	  if (lastloop >= 0) {
 	    for (i = lastloop; i >= 0; i--) {
 	      while (!loop[i] && i >= 0)
@@ -1467,16 +1663,15 @@ PyNetCDFVariable_WriteArray(PyNetCDFVariableObject *self,
 	    }
 	  }
 	}
-	if (self->unlimited) {
-	  Py_BEGIN_ALLOW_THREADS;
-	  acquire_netCDF_lock();
-	  ncdiminq(self->file->id, self->dimids[0], NULL,
-		   &self->dimensions[0]);
-	  release_netCDF_lock();
-	  Py_END_ALLOW_THREADS;
+	if (self->unlimited)
+	  nc_inq_dimlen(self->file->id, self->dimids[0], &self->dimensions[0]);
+	release_netCDF_lock();
+	Py_END_ALLOW_THREADS;
+	if (error != NC_NOERR) {
+	  netcdf_signalerror(error);
+	  ret = -1;
 	}
       }
-      Py_DECREF(array);
       if (start != NULL)
 	free(start);
       if (count != NULL)
@@ -1487,20 +1682,25 @@ PyNetCDFVariable_WriteArray(PyNetCDFVariableObject *self,
 	free(stride);
       if (current != NULL)
 	free(current);
+      if (loop != NULL)
+	free(loop);
     }
+    Py_DECREF(array);
     free(dims);
     free(indices);
     return ret;
   }
-  else
+  else {
+    free(dims);
+    free(indices);
     return -1;
+  }
 }
 
 static int
 PyNetCDFVariable_WriteString(PyNetCDFVariableObject *self,
 			     PyStringObject *value)
 {
-  long zero = 0;
   long len;
   if (self->type != PyArray_CHAR || self->nd != 1) {
     PyErr_SetString(PyExc_IOError, "netcdf: not a string variable");
@@ -1518,12 +1718,12 @@ PyNetCDFVariable_WriteString(PyNetCDFVariableObject *self,
     define_mode(self->file, 0);
     Py_BEGIN_ALLOW_THREADS;
     acquire_netCDF_lock();
-    ret = ncvarput(self->file->id, self->id, &zero, &len,
-		   PyString_AsString((PyObject *)value));
+    ret = nc_put_var_text(self->file->id, self->id,
+			  PyString_AsString((PyObject *)value));
     release_netCDF_lock();
     Py_END_ALLOW_THREADS;
-    if (ret == -1) {
-      netcdf_seterror();
+    if (ret != NC_NOERR) {
+      netcdf_signalerror(ret);
       return -1;
     }
     return 0;
@@ -1665,6 +1865,16 @@ PyNetCDFVariableObject_ass_slice(PyNetCDFVariableObject *self,
     PyErr_SetString(PyExc_TypeError, "Not a sequence");
     return -1;
   }
+  if (low < -(long)self->dimensions[0])
+    low = -self->dimensions[0];
+  if (low < 0)
+    low += self->dimensions[0];
+  if (high < low)
+    high = low;
+  if (self->unlimited && self->dimids[0] == self->file->recdim) {
+    if (high > self->dimensions[0])
+      high = self->dimensions[0];
+  }
   indices = PyNetCDFVariable_Indices(self);
   if (indices != NULL) {
     indices[0].start = low;
@@ -1830,7 +2040,7 @@ static PyMethodDef netcdf_methods[] = {
 
 /* Module initialization */
 
-void
+DL_EXPORT(void)
 initScientific_netcdf(void)
 {
   PyObject *m, *d;
